package dags;

import util.listreader.*;
import util.arrapp.Arrapp;
import java.io.*;
import java.util.regex.*;

public class DagMaker {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  static String inputExplanation =
    "Input format:\n" +
    "-b batchFile (-d dataFile)?\n\n" +
    "Use -b batchFile to pass a text file containing a list of definienda to be\n" +
    "processed, each in name:arity form. Output goes to a file named\n" +
    "batchFile_, in the tex directory.\n\n" +
    "-d dataFile names a file to which to append the computed data, or which to\n" +
    "create and use to store the computed data.";
  static String commandForm =
    "\\-b ((\\w|/|\\.)+)" +
    "(\\s\\-d ((\\w|/|\\.)+))?";
  static final int BATCHFILE = 1,
                   DATA = 4;
  static String batchFile, dataFile;
  
  static String[] tableData, dfds = null, arities;
  static FileOutputStream dataOut, texTableOut, texGraphOut;
  static int[][] dag;
  static DagGrapher dg;

  public static boolean verbose = true;

// ----------------------------------------------------------------------------
// Main
// ----------------------------------------------------------------------------

  public static void main(String[] args) {
    
    //Read the command line:
    String s = "";
    for (int i = 0; i < args.length; i++) {
      s += (i>0?" ":"");
      s += args[i];
    }
    Pattern commandPattern = Pattern.compile(commandForm);
    Matcher m = commandPattern.matcher(s);
    if (m.find()) {
      if (verbose) {
        int n = m.groupCount();
        for (int i = 0; i < n+1; i++) {
          System.out.println(
            Integer.toString(i)+": "+m.group(i)
          );
        }
      }
      batchFile = m.group(BATCHFILE);
      dataFile = m.group(DATA);
    } else {
      System.out.println(inputExplanation);
      return;
    }
    
    //Initialize according to command line:
    init();
    
    //Process the data:
    processData();
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  private static void processData() {
    String K,L,M;
    File f = new File(batchFile);
    String s;
    M = "\\bigskip\n\\begin{center}\n\\begin{tabular}{l c c | l c c}\n" +
                 "{\\bf " + batchFile + "}  \\\\\n" +
                 "{\\bf Definiendum} & {\\bf Depth} & {\\bf Size} & " +
                 "{\\bf Definiendum} & {\\bf Depth} & {\\bf Size} \\\\ \\hline \n";
    try {
      s = f.getName();
      s = s.substring(0,s.length()-2);
      texTableOut = new FileOutputStream("tex/"+s+"_DAGdata.tex");
      texTableOut.write(M.getBytes());
    } catch (IOException e) {}
    
    try {
      s = f.getName();
      s = s.substring(0,s.length()-2);
      texGraphOut = new FileOutputStream("tex/"+s+"_DAG.tex");
    } catch (IOException e) {}
        
    for (int i = 0; i < dfds.length; i++) {
      Dager.construct(dfds[i],arities[i]);
      dag = Dager.sortMulti();
      dg = new DagGrapher(dag);
      
      L = "$(" + dfds[i] + ":" + arities[i] + ")$\n";
      L += dg.graphMulti();
      L += "\n\n";
      M = "$\\mathop{\\mathtt{"+dfds[i] + "}}$:" + arities[i] + " & ";
      M += Integer.toString(dag.length) + " & ";
      M += Integer.toString(Dager.dfds.length);
      M += (i%2==0?" & ":"\\\\\n");
      K = Integer.toString(dag.length)+":"+
          Integer.toString(Dager.dfds.length);
      tableData = Arrapp.append(tableData,K);
      
      try {
        texGraphOut.write(L.getBytes());
        texTableOut.write(M.getBytes());
      } catch (IOException e) {}
    }
    
    M = "\\end{tabular}\n\\end{center}\n";
    try {
      texTableOut.write(M.getBytes());
    } catch (IOException e) {}
    
    if (dataFile!=null) {
      try {
        dataOut = new FileOutputStream(dataFile);
        for (int i = 0; i < tableData.length; i++) {
          K = tableData[i]+"\n";
          dataOut.write(K.getBytes());
        }
      } catch (IOException e) {}
    }
  }
  
  private static void init() {
    ListReader LR;
    
    if (dataFile!=null) {
      File f = new File(dataFile);
      if (f.exists()) {
        LR = new ListReader(dataFile);
        LR.initialize();
        tableData = LR.readAll();
      }
    }
    
    LR = new ListReader(batchFile);
    LR.initialize();
    dfds = LR.readAll();
    parseDfds();
  }
  
  static void parseDfds() {
    //Diagnostic:
    if (dfds==null) {System.out.println("<BatchDager.java:65>dfds is null.");}
    //
    arities = new String[dfds.length];
    String L; int w;
    for (int i = 0; i < dfds.length; i++) {
      L = dfds[i]; w = -1;
      for (int j = 0; j < L.length(); j++) {
        if (L.substring(j,j+1).equals(":")) {
          w = j; break;
        }
      }
      if (w==-1) {
        System.out.println("Error: user did not supply arity for dfd: " + L + ".");
      }
      dfds[i] = L.substring(0,w);
      arities[i] = L.substring(w+1,L.length());
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 