package dags;

import expand.ArgumentLocator;
import util.arrapp.Arrapp;
import org.w3c.dom.*;

public class Dager{

/** *****************************************************************
 *  The program operating at the next layer up (probably BatchDager)
 *  should call the construct method on the dfd and arity for which
 *  a DAG is desired.
 *  It can then call sortMulti and then the DAG will
 *  essentially be found in the four arrays, dfds, arities, levels, and parentsMulti.
 *  The sortMulti method returns an int[][] that gives the dag in levels.
 ******************************************************************** */

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  static Node N;
  static NodeList dfn;
  static int place = 0;
  static String[] dfds;
  static String[] arities;
  static int[] levels;
  static int[][] parentsMulti;
  
  static MathDefDictionary mdd = new MathDefDictionary();

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  public static void construct(String d, String a) {
    reset();
    getPlace(d,a);
    while (place < dfds.length) {
      N = mdd.lookUpDefiniens(dfds[place],arities[place]);
      dfn = N.getChildNodes();
      getChildren();
      place++;
    }
  }
  
  public static void reset() {
    place = 0; dfds = null; arities = null; levels = null; parentsMulti = null;
  }
  
  public static int[][] sortMulti() {
    //Diagnostic:
    System.out.println("Sorting multi.");
    //
    levels = new int[dfds.length];
    for (int i = 0; i < levels.length; i++) {levels[i] = -1;}
    levels[0] = 0;
    int nextLevel = 1;
    boolean done = false;
    boolean maybeDone = true;
    for (int i = 0; i < levels.length; i++) {maybeDone &= (levels[i]>=0);}
    done = maybeDone;
    while (!done) {
      for (int i = 0; i < dfds.length; i++) {
        if (levels[i]<0) {
          int[] P = parentsMulti[i];
          int[] L = new int[P.length];
          for (int j = 0; j < P.length; j++) {L[j] = levels[P[j]];}
          if (getMax(L)==nextLevel-1) {
            levels[i] = nextLevel;
          }
        }
      }
      nextLevel++;
      maybeDone = true;
      for (int i = 0; i < levels.length; i++) {maybeDone &= (levels[i]>=0);}
      done = maybeDone;
    }
    return collate();
  }
  
  static int getMax(int[] a) {
    int z = -1;
    if (a!=null) {
      for (int i = 0; i < a.length; i++) {
        if (a[i] > z) {z = a[i];}
        if (a[i] < 0) {z = -1; break;}
      }
    }
    return z;
  }
  
  static int[][] collate() {
    int n = levels.length;
    int nextLevel = 0;
    int[][] dag = new int[1][];
    dag[0] = new int[1];
    dag[0][0] = 0;
    n--;
    nextLevel++;
    while (n>0) {
      int[][] b = new int[dag.length+1][];
      System.arraycopy(dag,0,b,0,dag.length);
      dag = b;
      for (int i = 0; i < levels.length; i++) {
        if (levels[i]==nextLevel) {
          dag[nextLevel] = Arrapp.append(dag[nextLevel],i);
          n--;
        }
      }
      nextLevel++;
    }
    return dag;
  }
  
  static void getChildren() {
    Element e,d; String tag;
    for (int i = 0; i < dfn.getLength(); i++) {
      e = (Element) dfn.item(i);
      tag = e.getTagName();
      if (tag.equals("def")) {
        int[] endpts = new int[0];
        if (i<dfn.getLength()-1) {
          d = (Element) dfn.item(i+1);
          tag = d.getTagName();
          if (tag.equals("text")) {
            String u = d.getFirstChild().getNodeValue();
            if (u.equals("(") || u.equals("[")) {
              ArgumentLocator argloc = new ArgumentLocator(dfn,i+2);
              endpts = argloc.locate();
            }
          }
        }
        String s = e.getFirstChild().getNodeValue();
        String a = Integer.toString(endpts.length/2);
        int childPlace = getPlace(s,a);
        parentsMulti[childPlace] = Arrapp.union(parentsMulti[childPlace],place);
      }
    }
  }
  
  static int getPlace(String d, String a) {
  /** ***************************************************************
   *  Searches for (d,a) in (dfds,arities). If not found, it is added.
   *  In any case, this method returns the index at which (d,a) can
   *  (now) be found. Also updates the parentsMulti array as necessary
   *  to keep it the same length as the dfds and arities arrays.
   ****************************************************************** */
    int z = -1;
    if (dfds!=null && arities!=null) {
      for (int i = 0; i < dfds.length; i++) {
        if (dfds[i].equals(d) && arities[i].equals(a)) {
          z = i; break;
        }
      }
    }
    if (z<0) {
      dfds = Arrapp.append(dfds,d);
      arities = Arrapp.append(arities,a);
      extendPMulti();
      z = dfds.length - 1;
    }
    return z;
  }
  
  static void extendPMulti() {
    if (parentsMulti==null) {
      parentsMulti = new int[1][];
    } else {
      int[][] b = new int[parentsMulti.length+1][];
      System.arraycopy(parentsMulti,0,b,0,parentsMulti.length);
      parentsMulti = b;
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 