package expand;

import util.scanner.*;
import org.w3c.dom.*;

public class ArgumentLocator{
/** *******************************************************
 *  We assume that the boundaries of an argument will coincide
 *  with the boundaries of some XML tags. That is, we won't need
 *  to chop any <text> tag in half.
 ********************************************************** */

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  private NodeList nodelist;
  private int startPointer;
  private int pointer;
  
  private int parenlevel = 0;
  private int bracklevel = 0;
  private int bracelevel = 0;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public ArgumentLocator(NodeList nl, int sp) {
    /** ***************************************************
     *  sp should point to the first node *after* the opening
     *  parenthesis or bracket to the argument list.
     ****************************************************** */
    this.nodelist = nl;
    this.startPointer = sp;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public int[] locate() {
  /** *****************************************************
   *  
   ******************************************************** */
    int[] locations = new int[2];
    locations[0] = startPointer;
    pointer = startPointer;
    int[] ends = scan();
    locations[1] = ends[1];
    while (ends[2]!=-1) {
      ends = scan();
      locations = append(locations,ends[0]);
      locations = append(locations,ends[1]);
    }
    return locations;
  }
  
  private int[] scan() {
    int[] ends = new int[3];
    ends[0] = pointer;
    boolean done = false;
    Element e; String t;
    while (!done) {
      e = (Element) nodelist.item(pointer);
      //Diagnostic:
      if (e==null) {
      System.out.println("<ArgumentLocator:67>e is null");
      }
      if (nodelist==null) {
      System.out.println("<ArgumentLocator:67>nodelist is null");
      }
      //
      t = e.getTagName();
      if (t.equals("text")) {
      /** *************************************************
        * At this point we are looking for a character in
        * the text node that says that the argument has ended.
        * This will either be a comma occurring at level
        * zero, or else a close parenthesis or bracket that
        * brings us down to level -1.
        * By our assumptions about boundaries, this will mean
        * that the argument ended on the previous node.
        *************************************************** */
        String s = e.getFirstChild().getNodeValue();
        Scanner scanner = new Scanner(s);
        String[] r;
        boolean scanned = false;
        while (!scanned) {
          r = scanner.scan();
          if (r[2].equals("END")) {scanned = true;}
          if (r[1].equals("{")) {bracelevel++;}
          if (r[1].equals("(")) {parenlevel++;}
          if (r[1].equals("[")) {bracklevel++;}
          if (r[1].equals("}")) {bracelevel--;}
          if (r[1].equals(")")) {
            parenlevel--;
            if (parenlevel==-1) {
              ends[1] = pointer - 1;
              ends[2] = -1;
              done = true;
            }
          }
          if (r[1].equals("]")) {
            bracklevel--;
            if (bracklevel==-1) {
              ends[1] = pointer - 1;
              ends[2] = -1;
              done = true;
            }
          }
          if (r[1].equals(",") && zerolevel()) {
            ends[1] = pointer - 1;
            ends[2] = 0;
            done = true;
          }
        }
      }
      pointer++;
    }
    return ends;
  }
  
  private boolean zerolevel() {
    return (parenlevel==0) && (bracklevel==0) && (bracelevel==0);
  }
  
  private int[] append(int[] a, int z) {
    if (a==null) {
      a = new int[1];
      a[0] = z;
    } else {
      int[] b = new int[a.length+1];
      System.arraycopy(a,0,b,0,a.length);
      b[a.length] = z;
      a = b;
    }
    return a;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 