package expandDG;

import util.arrapp.Arrapp;

public class QbubbleMgr{

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  static final String OP = "(";
  static final String CP = ")";
  static final String BS = "\\";
  static final String FA = "forall";
  static final String EX = "exists";
  
  Qbubble[] bubs = new Qbubble[0];
  
  int deaths;
  int code;
  int pLevel;
  int qLevel;
  int qMax;
  
  int activeQ = 2;
  int depthType = 2; //2 for non-specific; 0 for sigma depth; 1 for pi depth
  
  public static boolean alternating = false;
  
  public static boolean verbose = false;
  
// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public QbubbleMgr() {
    pLevel = 0;
    qLevel = 0;
    qMax = 0;
    deaths = 0;
  }
  
  public QbubbleMgr(int depthType) {
    pLevel = 0;
    qLevel = 0;
    qMax = 0;
    deaths = 0;
    this.depthType = depthType;
    if (depthType!=2) {alternating = true;}
    resetActiveQ();
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  void setDepthType(int n) {
    depthType = n;
    resetActiveQ();
  }
  
  void resetActiveQ() {
    activeQ = depthType;
  }
  
  void read(String s) {
    int n = s.length();
    int ptr = 0;
    String c, r;
    while (ptr < n) {
      c = s.substring(ptr,ptr+1);
      ptr++;
      r = new String(c);
      if (c.equals(OP)) {
        op();
      } else if (c.equals(CP)) {
        cp();
      } else if (c.equals(BS)) {
        if (ptr+6<=n) {
          c = s.substring(ptr,ptr+6);
          if (c.equals(FA) || c.equals(EX)) {
            ptr += 6;
            r += new String(c);
            qu(c.equals(FA));
          } else {
            other();
          }
        } else {
          other();
        }
      } else {
        other();
      }
      if (verbose) {System.out.println("\nr:"+r);}
      passCodes();
      qLevel -= deaths;
      if (qLevel==0) {
        resetActiveQ();
      } else {
        if (deaths%2==1) {
          activeQ = 1 - activeQ;
        }
      }
      deaths = 0;
    }
  }

// --------------------------------------------------------

  void op() { //"Open paren"
    pLevel++;
    code = 0;
  }
  
  void cp() { //"Close paren"
    pLevel--;
    code = 1;
  }
  
  void qu(boolean type) { //"Quantifier"
    //Type true means it was a universal quantifier,
    //type false means it was an existential quantifier.
    Qbubble bub = new Qbubble(pLevel,Integer.toString(bubs.length));
    if ( !alternating ||
         activeQ == 2 ||
         (activeQ == 0 && !type) ||
         (activeQ == 1 && type)
       ) { bub.changer = true;
           if (type) {activeQ = 0;} else {activeQ = 1;}
         }
    addBub(bub);
    qLevel += bub.change();
    if (qLevel > qMax) {qMax = qLevel;}
    code = 2;
  }
  
  void other() {
    code = 3;
  }
  
// --------------------------------------------------------

  void passCodes() {
    if (verbose) {
      System.out.println("Q:"+Integer.toString(qLevel)+" "+
                         "P:"+Integer.toString(pLevel)+" "+
                         "code:"+Integer.toString(code)+"."
      );
    }
    deaths = 0;
    for (int i = 0; i < bubs.length; i++) {
      deaths += bubs[i].codeCall(pLevel,code);
    }
    removeBubs();
  }
  
  void addBub(Qbubble b) {
    if (bubs==null) {
      bubs = new Qbubble[1];
      bubs[0] = b;
    } else {
      Qbubble[] x = new Qbubble[bubs.length+1];
      System.arraycopy(bubs,0,x,0,bubs.length);
      x[bubs.length] = b;
      bubs = x;
    }
  }
  
  void removeBubs() {
    int k = 0;
    while (k < bubs.length) {
      if (bubs[k].dead) {
        if (bubs.length==1) {
          bubs = new Qbubble[0];
        } else {
          Qbubble[] a = new Qbubble[bubs.length-1];
          System.arraycopy(bubs,0,a,0,k);
          System.arraycopy(bubs,k+1,a,k,a.length-k);
          bubs = a;
        }  
      } else {
        k++;
      }
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 