package expandDG;

import util.listreader.*;
import util.arrapp.Arrapp;
import java.io.*;

public class Qstats{

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  static final String DATAFILE = "expandDG/Qdata";
  static final String OUTNAME = "expandDG/Qstat.txt";
  
  public static boolean verbose = true;
  
  static String[] dataLines;
  static int[][] data;
  static FileOutputStream out;

// ----------------------------------------------------------------------------
// Main
// ----------------------------------------------------------------------------

  public static void main(String[] args) {
    try {
      ListReader LR = new ListReader(DATAFILE);
      LR.initialize();
      dataLines = LR.readAll();
      File f = new File(OUTNAME);
      out = new FileOutputStream(f);
      //
      for (int i = 0; i < dataLines.length; i++) {
        data = Arrapp.append(data,parse(dataLines[i]));
      }
      //diag:
      if (verbose) {
        String s;
        for (int i = 0; i < data.length; i++) {
          s = "";
          for (int j = 0; j < data[i].length; j++) {
            s += (j>0?":":"")+Integer.toString(data[i][j]);
          }
          //System.out.println(s);
        }
        int c = Integer.parseInt(args[0]);
        int p = Integer.parseInt(args[1]);
        System.out.println(
          "Max for "+args[0]+": "+Integer.toString(maximum(c))+"\n"+
          "Mean for "+args[0]+": "+Float.toString(mean(c))+"\n"+
          "MaxDiff for "+args[0]+" and "+args[1]+": "+
            Integer.toString(maxDiff(c,p))+"\n"+
          "MeanDiff for "+args[0]+" and "+args[1]+": "+
            Float.toString(meanDiff(c,p))
        );
        int[][] freq = frequencies(c);
        System.out.println("Frequencies for "+args[0]+":");
        for (int i = 0; i < freq[0].length; i++) {
          System.out.println(
            Integer.toString(freq[0][i])+","+
            Integer.toString(freq[1][i])
          );
        }
      }
      //
      int[][] freq = new int[0][];
      for (int i = 0; i < 8; i++) {
        int[][] zz = new int[freq.length+2][];
        System.arraycopy(freq,0,zz,0,freq.length);
        int[][] yy = frequencies(i);
        System.arraycopy(yy,0,zz,freq.length,2);
        freq = zz;
      }
      int M = 0;
      for (int i = 0; i < 8; i++) {
        if (freq[2*i].length>M) {M=freq[2*i].length;}
      }
      String s;
      for (int j = 0; j < M; j++) {
        s = "";
        for (int i = 0; i < 8; i++) {
          if (j<freq[2*i].length) {
            s += Integer.toString(freq[2*i][j]);
            s += ",";
            s += Integer.toString(freq[2*i+1][j]);
            s += (i<7?",":"\n");
          } else {
            s += " , "+(i<7?",":"\n");
          }
        }
        out.write(s.getBytes());
      }
    } catch (IOException e) {e.printStackTrace();}
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  /**
   * Summary:
   *   maximum
   *   maxDiff
   *   mean
   *   meanDiff
   *   frequencies
   */
  
  
  static int maximum(int c) {
    //c must be between 0 and 7, inclusive.
    int M = 0;
    for (int i = 0; i < data.length; i++) {
      if (data[i][c]>M) {M = data[i][c];}
    }
    return M;
  }
  
  static int maxDiff(int c, int d) {
    int M = 0;
    int delta;
    for (int i = 0; i < data.length; i++) {
      delta = Math.abs(data[i][c]-data[i][d]);
      if (delta>M) {M = delta;}
    }
    return M;
  }
  
  static float mean(int c) {
    float T = 0;
    for (int i = 0; i < data.length; i++) {
      T += data[i][c];
    }
    return T/data.length;
  }
  
  static float meanDiff(int c, int d) {
    float T = 0;
    for (int i = 0; i < data.length; i++) {
      T += Math.abs(data[i][c]-data[i][d]);
    }
    return T/data.length;
  }
  
  static int[][] frequencies(int c) {
    int[] z = new int[1];
    z[0] = data[0][c];
    int[] n = new int[1];
    n[0] = 1;
    //
    int w;
    for (int i = 1; i < data.length; i++) {
      w = findInt(z,data[i][c]);
      if (w>=0) {
        n[w]++;
      } else {
        w = findFirstExceeding(z,data[i][c]);
        if (w>=0) {
          int[] a = new int[z.length+1];
          System.arraycopy(z,0,a,0,w);
          a[w] = data[i][c];
          System.arraycopy(z,w,a,w+1,z.length-w);
          System.arraycopy(n,0,a,0,w);
          a[w] = 1;
          System.arraycopy(n,w,a,w+1,n.length-w);
        } else {
          z = Arrapp.append(z,data[i][c]);
          n = Arrapp.append(n,1);
        }
      }
    }
    //
    int[][] R = new int[2][];
    R[0] = z;
    R[1] = n;
    return R;
  }
  
  static int findFirstExceeding(int[] a, int z) {
    int w = -1;
    for (int i = 0; i < a.length; i++) {
      if (a[i]>z) {w = i; break;}
    }
    return w;
  }
  
  static int findInt(int[] a, int z) {
    int w = -1;
    for (int i = 0; i < a.length; i++) {
      if (a[i]==z) {w = i; break;}
    }
    return w;
  }
  
  static int[] parse(String s) {
    int[] a = null;
    int[] b = new int[1];
    b[0] = -1;
    for (int w = 0; w < s.length(); w++) {
      if (s.substring(w,w+1).equals(":")) {
        b = Arrapp.append(b,w);
      }
    }
    b = Arrapp.append(b,s.length());
    for (int j = 0; j < b.length - 1; j++) {
      a = Arrapp.append(a,Integer.parseInt(s.substring(b[j]+1,b[j+1])));
    }
    return a;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 