package expandDG;

import util.arrapp.Arrapp;
import expand.ArgumentLocator;
import org.w3c.dom.*;

public class QuantDG extends ListDriven {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  static final String QFUNCS0 = "expandDG/qfuncs0";
  static final String QFUNCSA = "expandDG/qfuncsA";
  static final String QFUNCSB = "expandDG/qfuncsB";
  
  static int mode = 0;
  
  static String dfd, arity;
  static NodeList dfn;
  
  static String[] leaves;
  static int[] leafArities;
  
  static final long LIMIT = Integer.MAX_VALUE;
  
  static boolean verbose = false;
  
  static int degree = 0;
  
  int[] function;
  QbubbleMgr qbm;

// --------------------------------------------------------
// Main
// --------------------------------------------------------

  public static void main(String[] args) {
  }

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------

  public QuantDG() {
    function = new int[degree+1];
    for (int i = 0; i < degree+1; i++) {function[i]=0;}
    if (verbose) {vox(0);}
    qbm = new QbubbleMgr();
    QbubbleMgr.alternating = false;
  }

// --------------------------------------------------------
// Vox
// --------------------------------------------------------

  void vox(int n) {vox("",n);}
  
  void vox(String t, int n) {
    switch(n) {
      case 0:
        String S = t + Integer.toString(function[0]);
        for (int j = 1; j < function.length; j++) {
          S += ","+Integer.toString(function[j]);
        }
        System.out.println(S);
      break;
    }
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------
  
  int[] buildFunc() {return buildFunc(0,dfn.getLength()-1);}
  
  int[] buildFunc(int start, int stop) {
    Element e, d; String tag, t, n, s;
    for (int i = start; i <= stop; i++) {
      e = (Element) dfn.item(i);
      tag = e.getTagName();
      if (tag.equals("text")) {
        s = e.getFirstChild().getNodeValue();
        text(s);
      } else if (tag.equals("var")) {
        t = e.getAttribute("t");
        n = e.getAttribute("n");
        var(t,n);
      } else if (tag.equals("def")) {
        int[] endpts = new int[0];
        d = (Element) dfn.item(i+1);
        if (d!=null) {
          tag = d.getTagName();
        }
        if (tag.equals("text")) {
          String u = d.getFirstChild().getNodeValue();
          if (u.equals("(") || u.equals("[")) {
            ArgumentLocator argloc = new ArgumentLocator(dfn,i+2);
            endpts = argloc.locate();
          }
        }
        s = e.getFirstChild().getNodeValue();
        int a = endpts.length/2;
        if (mode==0 || (mode==2 && searchLeaves(s,a)>=0)) {
          text(s);
        } else {
          if (endpts.length>0) {
            i = endpts[endpts.length-1]+1;
          }
          try {
            int[] F = def(s,endpts);
            F = cappedAdd(F,qbm.qLevel);
            function = maxAgainst(function,F);
            if (verbose) {vox("exp:",0);}
          } catch (Exception ex) {
            ex.printStackTrace();
          }
        }
      }
    }
    function[degree] = Math.max(function[degree],qbm.qMax);
    if (verbose) {vox(0);}
    return function;
  }
  
  int[] def(String s, int[] endpts) throws Exception {
    int g = endpts.length/2;
    int[][] argFuncs = new int[0][];
    QuantDG qdg;
    int[] F;
    for (int i = 0; i < g; i++) {
      qdg = new QuantDG();
      F = qdg.buildFunc(endpts[2*i],endpts[2*i+1]);
      argFuncs = Arrapp.append(argFuncs,F);
    }
    int[] H = new int[0];
    int h = searchFuncs(s,g);
    if (h<0) {
      throw new Exception("Couldn't find function for "+s+":"+
                          Integer.toString(g));
    } else {
      H = functions[h];
    }
    return compose(H,argFuncs);
  }
  
  void text(String s) {
    qbm.read(s);
  }
  
  void var(String t, String n) {
    if (t.equals("f")) {
      int N = Integer.parseInt(n);
      function[N] = Math.max(function[N],qbm.qLevel);
      if (verbose) {vox("var:",0);}
    }
  }
  
  static int evalAtZeros(int[] F) {
    return maximum(F);
  }
  
  static int[] compose(int[] H, int[][] A) {
    int n = degree+1;
    try {
      if (H.length-1<A.length) {
        throw new Exception("Trying to compose wrong arities!"+
                            Integer.toString(H.length-1)+" and "+
                            Integer.toString(A.length));
      }
      boolean okay = true;
      for (int i = 0; i < A.length; i++) {
        if (A[i].length!=n) {okay = false;}
      }
      if (!okay) {
        throw new Exception("Mixed input arities!");
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    int[] F = new int[n];
    int[] c;
    for (int i = 0; i < n; i++) {
      c = new int[A.length];
      for (int j = 0; j < A.length; j++) {
        c[j] = cappedAdd(H[j],A[j][i]);
      }
      F[i] = maximum(c);
    }
    F[n-1] = Math.max(F[n-1],H[H.length-1]);
    return F;
  }
  
  static int maximum(int[] a) {
    int M = 0;
    if (a!=null) {
      for (int i = 0; i < a.length; i++) {
        if (a[i]>M) {M = a[i];}
      }
    }
    return M;
  }
  
  static int searchFuncs(String s, int g) {
    int location = -1;
    if (dfds!=null) {
      for (int i = 0; i < dfds.length; i++) {
        if (dfds[i].equals(s) && arities[i].equals(Integer.toString(g))) {
          location = i; break;
        }
      }
    }
    return location;
  }
  
  static int cappedAdd(int a, int b) {
    long A = (long) a;
    long B = (long) b;
    A += B;
    if (A>LIMIT) {
      return (int) LIMIT;
    } else {
      return (int) A;
    }
  }
  
  static int[] cappedAdd(int[] a, int z) {
    for (int i = 0; i < a.length; i++) {
      a[i] = cappedAdd(a[i],z);
    }
    return a;
  }
  
  static int cappedMult(int m, int n) {
    long M = (long) m;
    long N = (long) n;
    M *= N;
    if (M>LIMIT) {
      return (int) LIMIT;
    } else {
      return (int) M;
    }
  }
  
  static int[] cappedAdd(int[] a, int[] b) {
    try {
      if (a.length!=b.length) {
        throw new Exception("Adding arrays of different lengths!");
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    for (int i = 0; i < a.length; i++) {
      a[i] = cappedAdd(a[i],b[i]);
    }
    return a;
  }
  
  static int[] maxAgainst(int[] a, int[] b) {
    try {
      if (a.length!=b.length) {
        throw new Exception("Maxing arrays of different lengths!");
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    for (int i = 0; i < a.length; i++) {
      a[i] = Math.max(a[i],b[i]);
    }
    return a;
  }
  
  static int searchLeaves(String s, int a) {
    int location = -1;
    if (leaves!=null) {
      for (int i = 0; i < leaves.length; i++) {
        if (leaves[i].equals(s) && leafArities[i]==a) {
          location = i; break;
        }
      }
    }
    return location;
  }
  
  static void setList() {
    switch (mode) {
      case 0: list = QFUNCS0; break;
      case 1: list = QFUNCSA; break;
      case 2: list = QFUNCSB; break;
    }
  }
  
  static void preScan() throws Exception {
  /**
   * Figures out how many free variables are in the dfn, and
   * sets the length of 'function' accordingly.
   */
    int[] A = new int[0];
    Element e; String tag, t, n; int N;
    for (int i = 0; i < dfn.getLength(); i++) {
      e = (Element) dfn.item(i);
      tag = e.getTagName();
      if (tag.equals("var")) {
        t = e.getAttribute("t");
        if (t.equals("f")) {
          n = e.getAttribute("n");
          N = Integer.parseInt(n);
          A = Arrapp.union(A,N);
        }
      }
    }
    //diag
    if (verbose) {
      String L = "";
      for (int i = 0; i < A.length; i++) {
        L += (i==0?"":",");
        L += Integer.toString(A[i]);
      }
      System.out.println("Free var numbers: " + L);
    }
    //
    //Check that only consecutive numbers, starting with 0, have been
    //used:
    boolean okay = true;
    for (int i = 0; i < A.length; i++) {
      if (A[i]>=A.length) {okay = false;}
    }
    if (!okay) {
      throw new Exception("On "+dfd+":"+arity+
                          ", free vars have bad numbers!");
    } else {
      degree = A.length;
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 