package expandDG;

import util.listreader.*;
import util.arrapp.Arrapp;
import org.w3c.dom.*;
import java.io.*;
import java.util.regex.*;

public class SymbolCounter {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  static String inputExplanation =
    "Input format:\n" +
    "[name:arity | -b batchFile (outputFile)?] (-c)? " +
    "(-l leafFile)? (-d dataFile)?\n\n" +
    "Use name:arity to input a single definiendum. Output is to screen.\n\n" +
    "Use -b batchFile to pass a text file containing a list of definienda to be\n" +
    "processed, each in name:arity form. Output goes to a file named\n" +
    "batchFile_SymbolData.tex, in the tex directory, unless you specify an output file name.\n\n" +
    "Use -c if you want to clear the files containing the coefficients for the\n" +
    "depth functions.\n\n" +
    "-l leafFile specifies definienda that should not be expanded.\n\n" +
    "-d dataFile names a file to which to append the computed data, or which to\n" +
    "create and use to store the computed data.";
  static String commandForm =
    "(\\-b ((\\w|/|\\.)+)( ((\\w|/|\\.)+))?|((\\w|\\\\|\\{|\\})+):((\\d)+))" +
    "(\\s\\-c)?" +
    "(\\s\\-l ((\\w|/|\\.)+))?" +
    "(\\s\\-d ((\\w|/|\\.)+))?";
  static final int BATCHFILE = 2,
                   OUTPUTFILE = 5,
                   DFD = 7,
                   ARITY = 9,
                   CLEAR = 11,
                   LEAVES = 13,
                   DATA = 16;
  static String batchFile, outputFile = null, dfd,
                arity, clear, leafFile, dataFile;
  
  static String[] leaves, tableData, dfds = null, arities;
  static String[][] data;
  static int[] leafArities;
  static FileOutputStream dataOut, texOut;
  static Node N;
  static NodeList dfn;
  static MathDefDictionary mdd = new MathDefDictionary();
  
  public static boolean verbose = true;

// ----------------------------------------------------------------------------
// Main
// ----------------------------------------------------------------------------

/** ***********************************************************
 *  Input options:
 *  - batch operation vs. single inquiry : -b MunkTop2 / ANSYM:2
 *  - leaf file : -l leafFile
 *  - append data to comma-delimited file, or start new file : -d dataFile
 *  - clear coefficient files : -c
 ************************************************************* */

  public static void main(String[] args) {
    
    //Read the command line:
    String s = "";
    for (int i = 0; i < args.length; i++) {
      s += (i>0?" ":"");
      s += args[i];
    }
    Pattern commandPattern = Pattern.compile(commandForm);
    Matcher m = commandPattern.matcher(s);
    if (m.find()) {
      if (verbose) {
        int n = m.groupCount();
        for (int i = 0; i < n+1; i++) {
          System.out.println(
            Integer.toString(i)+": "+m.group(i)
          );
        }
      }
      batchFile = m.group(BATCHFILE);
      outputFile = m.group(OUTPUTFILE);
      dfd = m.group(DFD);
      arity = m.group(ARITY);
      clear = m.group(CLEAR);
      leafFile = m.group(LEAVES);
      dataFile = m.group(DATA);
    } else {
      System.out.println(inputExplanation);
      return;
    }
    
    //Initialize according to command line:
    init();
    
    //Process the data:
    getData();
    
    //Output:
    output();
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  private static void output() {
    if (dfd!=null) {
      System.out.println(
        "DZFC: " + data[0][0] + "\n" +
        "DZFC fully expanded: " + data[0][1] + "\n" +
        "DZFC partially expanded: " + data[0][2]
      );
    } else {
      if (outputFile==null) {
        File f = new File(batchFile);
        String u = f.getName();
        u = u.substring(0,u.length()-2);
        outputFile = "tex/"+u+"_SymbolData.tex";
      }
      try {
        texOut = new FileOutputStream(outputFile);
        String S = "\\bigskip\n"+
            "\\begin{center}\n"+
            "\\begin{tabular}{|l|c c c|}\n"+
            "\\multicolumn{4}{l}{{\\bf "+batchFile+"}} \\\\\\hline\n"+
            "{\\bf Definiendum}\n"+
            "& $\\alpha$ & $\\mathscr{E}(\\alpha,\\varnothing)$ "+
            "& $\\mathscr{E}(\\alpha,B_0)$\\\\ \\hline\n";
        texOut.write(S.getBytes());
        for (int i = 0; i < dfds.length; i++) {
          S = "$\\mathop{\\mathtt{"+dfds[i]+"}}$:"+arities[i]+" & "+
              data[i][0]+" & "+
              data[i][1]+" & "+
              data[i][2]+"\\\\\n";
          texOut.write(S.getBytes());
        }
        S = "\\hline\n"+
            "\\end{tabular}\n"+
            "\\end{center}\n";
        texOut.write(S.getBytes());
      } catch (IOException e) {}
    }
    
    if (dataFile!=null) {
      String T;
      for (int i = 0; i < dfds.length; i++) {
        T = "";
        for (int j = 0; j < 3; j++) {
          T += data[i][j] + (j<2?",":"");
        }
        tableData = Arrapp.append(tableData,T);
      }
      try {
        File f = new File(dataFile);
        dataOut = new FileOutputStream(f);
        for (int i = 0; i < tableData.length; i++) {
          T = tableData[i] + "\n";
          dataOut.write(T.getBytes());
        }
      } catch (IOException e) {e.printStackTrace();}
    }
  }
  
  private static void getData() {
    LengthDG ldg; int[] F; int D; String S;
    for (int k = 0; k < 3; k++) {
      LengthDG.mode = k;
      LengthDG.setList();
      if (clear==null) {
        LengthDG.init();
      }
      for (int i = 0; i < dfds.length; i++) {
        N = mdd.lookUpDefiniens(dfds[i],arities[i]);
        dfn = N.getChildNodes();
        LengthDG.dfd = dfds[i]; LengthDG.arity = arities[i];
        LengthDG.dfn = dfn;
        try {
          LengthDG.preScan();
        } catch (Exception e) {
          e.printStackTrace();
        }
        ldg = new LengthDG();
        F = ldg.buildFunc();
        LengthDG.functions = Arrapp.append(LengthDG.functions,F);
        LengthDG.dfds = Arrapp.append(LengthDG.dfds,dfds[i]);
        String A = (arities[i].equals("infix")?"2":arities[i]);
        LengthDG.arities = Arrapp.append(LengthDG.arities,A);
        D = LengthDG.evalAtOnes(F);
        S = Integer.toString(D);
        if (D==Integer.MAX_VALUE) {S += "+";}
        data[i][k] = new String(S);
      }
      LengthDG.record();
    }
  }
  
  private static void init() {
    ListReader LR;
    
    if (leafFile!=null) {
      LR = new ListReader(leafFile);
      LR.initialize();
      leaves = LR.readAll();
      parseLeaves();
      LengthDG.leaves = leaves;
      LengthDG.leafArities = leafArities;
    }
    
    if (dataFile!=null) {
      File f = new File(dataFile);
      if (f.exists()) {
        LR = new ListReader(dataFile);
        LR.initialize();
        tableData = LR.readAll();
      }
    }
    
    if (dfd!=null && arity!=null) {
      dfds = new String[1];
      dfds[0] = dfd;
      arities = new String[1];
      arities[0] = arity;
    } else if (batchFile!=null) {
      LR = new ListReader(batchFile);
      LR.initialize();
      dfds = LR.readAll();
      parseDfds();
    }
    
    data = new String[dfds.length][];
    for (int i = 0; i < dfds.length; i++) {
      data[i] = new String[3];
    }
  }
  
  static void parseLeaves() {
    if (leaves!=null) {
      leafArities = new int[leaves.length];
      String L; int w;
      for (int i = 0; i < leaves.length; i++) {
        L = leaves[i]; w = -1;
        for (int j = 0; j < L.length(); j++) {
          if (L.substring(j,j+1).equals(":")) {
            w = j; break;
          }
        }
        if (w==-1) {
          System.out.println("Error: user did not supply arity for leaf: " + L + ".");
        }
        leaves[i] = L.substring(0,w);
        leafArities[i] = Integer.parseInt(L.substring(w+1,L.length()));
      }
    }
  }
  
  static void parseDfds() {
    //Diagnostic:
    if (dfds==null) {System.out.println("<BatchDager.java:65>dfds is null.");}
    //
    arities = new String[dfds.length];
    String L; int w;
    for (int i = 0; i < dfds.length; i++) {
      L = dfds[i]; w = -1;
      for (int j = 0; j < L.length(); j++) {
        if (L.substring(j,j+1).equals(":")) {
          w = j; break;
        }
      }
      if (w==-1) {
        System.out.println("Error: user did not supply arity for dfd: " + L + ".");
      }
      dfds[i] = L.substring(0,w);
      arities[i] = L.substring(w+1,L.length());
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 