package graphs;

import java.awt.Dimension;
import java.awt.Point;
import javax.swing.JLabel;

public class Vertex {

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  int X, Y;
  protected String text;
  Edge[] edges;
  protected Dimension size;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public Vertex() {
  }
  
  public Vertex(int x, int y) {
    X = x; Y = y;
  }
  
  public Vertex(String t) {
    text = t;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  void addEdge(Edge e) {
    if (edges==null) {
      edges = new Edge[1];
      edges[0] = e;
    } else {
      Edge[] b = new Edge[edges.length+1];
      System.arraycopy(edges,0,b,0,edges.length);
      b[edges.length] = e;
      edges = b;
    }
  }
  
  void removeEdge(Edge e) {
  /** This method assumes there are no repeats in the edges array. */
    if (edges!=null) {
      boolean present = false;
      for (int i = 0; i < edges.length; i++) {
        present |= (edges[i].equals(e));
      }
      if (present) {
        Edge[] f = new Edge[edges.length-1];
        int passed = 0;
        for (int i = 0; i < edges.length; i++) {
          if (edges[i].equals(e)) {
            passed = 1;
          } else {
            f[i-passed] = edges[i];
          }
        }
        edges = f;
      }
    }
  }
  
  Edge[] getEdges() {return edges;}
  
  String getText() {return text;}
  
  Point getPosition() {return new Point(X,Y);}
  
  void setText(String s) {
    text = s;
  }
  
  void setPosition(int x, int y) {
    X = x; Y = y;
  }
  
  void setPosition(Point p) {
    X = p.x; Y = p.y;
  }
  
  boolean overlaps(Vertex v) {
    if (size==null) {getSize();}
    int W = size.width; int H = size.height;
    int x = v.X; int y = v.Y;
    int w = v.getSize().width;
    int h = v.getSize().height;
    return (x-W<=X) && (X<=x+w) && (y-h<=Y) && (Y<=y+H);
  }
  
  boolean isOver(int x, int y) {
    int w, h;
    if (size==null) {getSize();}
    w = size.width; h = size.height;
    return (X<=x) && (x<=X+w) && (Y-h<=y) && (y<=Y);
  }
  
  Dimension getSize() {
    if (size!=null) {
      return size;
    } else {
      if (text==null) {
        return null;
      } else {
        JLabel temp = new JLabel(text);
        size = temp.getPreferredSize();
        return size;
      }
    }
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 