package lpt2dzfc;

public class Set{

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------

  public FormulaBuilder term;
  public FormulaBuilder[] merelist = null;
  public Infixrel precondrel = null;
  public FormulaBuilder[] precondterms = null;
  public FormulaBuilder suchthat;
  public String[] fixed = null;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public Set() {
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public FormulaBuilder output() {
    FormulaBuilder f;
    if (merelist!=null) {
      FormulaBuilder[] a = new FormulaBuilder[merelist.length+1];
      a[0] = term;
      System.arraycopy(merelist,0,a,1,merelist.length);
      f = setOf(a);
    }
    else {f = complex();}
    return f;
  }
  
  private FormulaBuilder complex() {
    FormulaBuilder f = new FormulaBuilder();
    FormulaBuilder d = definingConds(this.term, this.suchthat,
                                     this.precondrel, this.precondterms,
                                     this.fixed);
    String set = d.fresh();
    f.addText("(\\iota ");
    f.addVar(set);
    f.addText(")(\\forall ");
    f.addVar(d.auxData[0]);
    f.addText(")(");
    f.addVar(d.auxData[0]);
    f.addText(" \\in ");
    f.addVar(set);
    f.addText(" \\rightleftarrow ");
    f.append(d);
    f.addText(")");
    f.bind(d.auxData[0]);
    f.bind(set);
    return f;
  }
  
  public FormulaBuilder definingConds(FormulaBuilder term, FormulaBuilder suchthat,
                                       Infixrel precondrel, FormulaBuilder[] precondterms,
                                       String[] fixed) {
    FormulaBuilder dc = new FormulaBuilder();
    //Check:
    if (fixed!=null) {
      boolean okay = true;
      int z;
      for (int i = 0; i < fixed.length; i++) {
        z = term.findString(term.f,fixed[i]);
        if (z<0) {okay = false;}
      }
      if (!okay) {dc.addText("Fixed vars must be free in main term.");}
    }
    //Generate fresh var:
    FormulaBuilder g = new FormulaBuilder();
    g.append(term); g.append(suchthat);
    if (precondterms!=null) {
      for (int i = 0; i < precondterms.length; i++) {g.append(precondterms[i]);}
    }
    String y = g.fresh();
    dc.auxData = new String[1];
    dc.auxData[0] = y;
    //Determine vars to be existentially quantified:
    String[] exquan = new String[0];
    if (fixed==null) {exquan = term.f;}
    else {
      int w;
      for (int i = 0; i < term.f.length; i++) {
        w = term.findString(fixed,term.f[i]);
        if (w<0) {exquan = term.concatString(exquan,term.f[i]);}
      }
    }
    //Build formula:
    if (exquan.length>0) {
      dc.addText("(\\exists ");
      for (int i = 0; i < exquan.length; i++) {
        dc.addVar(exquan[i]);
        if (i!=exquan.length-1) {dc.addText(",");}
      }
      dc.addText(")");
    }
    dc.addText("(");
    dc.addVar(y);
    dc.addText(" = ");
    dc.append(term);
    dc.addText(" \\wedge (");
    dc.append(suchthat);
    dc.addText(")");
    if (precondterms!=null) {
      for (int i = 0; i < precondterms.length; i++) {
        dc.addText(" \\wedge (");
        dc.append(precondrel.output(term,precondterms[i]));
        dc.addText(")");
      }
    }
    dc.addText(")");
    for (int i = 0; i < exquan.length; i++) {
      dc.bind(exquan[i]);
    }
    return dc;
  }
  
  public FormulaBuilder setOf(FormulaBuilder[] e) {
    //Generate fresh vars:
    FormulaBuilder g = new FormulaBuilder();
    for (int i = 0; i < e.length; i++) {g.append(e[i]);}
    FormulaBuilder set = new FormulaBuilder();
    String setString = g.fresh();
    set.addVar(setString);
    g.append(set);
    FormulaBuilder el = new FormulaBuilder();
    String elString = g.fresh();
    el.addVar(elString);
    //Build formula:
    FormulaBuilder S = new FormulaBuilder();
    S.addText("(\\iota ");
    S.append(set);
    S.addText(")(\\forall ");
    S.append(el);
    S.addText(")(");
    S.append(el);
    S.addText(" \\in ");
    S.append(set);
    S.addText(" \\rightleftarrow ");
    for (int i = 0; i < e.length; i++) {
      S.append(el);
      S.addText(" = ");
      S.append(e[i]);
      if (i!=e.length-1) {S.addText(" \\vee ");}
    }
    S.addText(")");
    S.bind(setString);
    S.bind(elString);
    return S;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 