package util.listreader;

import java.io.*;

public class ListReader{

// ----------------------------------------------------------------------------
// Data
// ----------------------------------------------------------------------------
  
  private String listFileName = null;
  private File listFile = null;
  private FileInputStream list = null;
  private int nextInt = 33; //Exclamation mark in case
                            //this fails to be initialized
  private boolean primed = false;
  private boolean ready = false;

// ----------------------------------------------------------------------------
// Constructors
// ----------------------------------------------------------------------------

  public ListReader() {
  }
  
  public ListReader(String s) {
    //Diagnostic:
    System.out.println("<ListReader.java:28>ListReader has been constructed on string: " + s);
    //
    listFileName = s;
  }
  
  public ListReader(File f) {
    listFile = f;
  }
  
  public ListReader(FileInputStream in) {
    list = in;
  }

// ----------------------------------------------------------------------------
// Methods
// ----------------------------------------------------------------------------

  public void setListFileName(String s) {
    listFileName = s;
    listFile = null;
    list = null;
    ready = false;
    primed = false;
  }
  
  public void setListFile(File f) {
    listFileName = null;
    listFile = f;
    list = null;
    ready = false;
    primed = false;
  }
  
  public void setList(FileInputStream in) {
    listFileName = null;
    listFile = null;
    list = in;
    ready = true;
    primed = false;
  }
  
  public String getListFileName() {
    return listFileName;
  }
  
  public File getListFile() {
    return listFile;
  }
  
  public FileInputStream getList() {
    return list;
  }
  
  public int initialize() {
    int n = readyList();
    if (n==0) { prime(); }
    return n;
  }
  
  private int readyList() {
    if (ready) {
      return 0;
    } else {
      if (list != null) {
        ready = true;
        return 0;
      } else {
        if (listFile == null) {
          if (listFileName == null) {return 1;}
          else {
            listFile = new File(listFileName);
          }
        }
        try {
          list = new FileInputStream(listFile);
        } catch (IOException ioe) {
          return 2;
        }
        ready = true;
        return 0;
      }
    }
  }
  
  private int prime() {
    int n = 0;
    if (ready && !primed) {
      try {
        nextInt = list.read();
      } catch (IOException ioe) {
        n = 2;
      }
      primed = true;
    }
    return n;
  }
  
  public String read() {
    if (primed && ready) {
      StringBuffer sb = new StringBuffer();
      int b = nextInt;
      if (b == -1) {return null;}
      else {
        while (b != 10 && b != 13 && b != -1) {
          sb.append((char) b);
          try {
            b = list.read();
          } catch (IOException ioe) {
            return null;
          }
        }
        String s = new String(sb);
        //Advance pointer to beginning of next word:
        while (b == 10 || b == 13) {
          try {
            b = list.read();
          } catch (IOException ioe) {
            return null;
          }
        }
        nextInt = b;
        return s;
      }
    } else { return null; }
  }
  
  public String[] readAll() {
    if (primed && ready) {
      String[] a = new String[0];
      String s = read();
      //Diagnostic:
      System.out.println("<ListReader.java:159>ListReader just read: " + s);
      //
      while (s!=null) {
        a = append(a,s);
        s = read();
      }
      return a;
    } else { return null; }
  }
  
  public String[] append(String[] a, String s) {
    if (a==null) {
      a = new String[1];
      a[0] = s;
    } else {
      String[] b = new String[a.length + 1];
      System.arraycopy(a,0,b,0,a.length);
      b[a.length] = s;
      a = b;
    }
    return a;
  }

// ----------------------------------------------------------------------------
}  // ----------------------------------------------------------------------
// ---------------------------------------------------------------------------- 